"use strict";
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __name = (target, value) => __defProp(target, "name", { value, configurable: true });
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);

// src/index.ts
var src_exports = {};
__export(src_exports, {
  Button: () => Button,
  Command: () => Command,
  Event: () => Event,
  JellyCommands: () => JellyCommands,
  MessageCommand: () => MessageCommand,
  Modal: () => Modal,
  UserCommand: () => UserCommand,
  button: () => button,
  command: () => command,
  event: () => event,
  messageCommand: () => messageCommand,
  modal: () => modal,
  userCommand: () => userCommand
});
module.exports = __toCommonJS(src_exports);

// src/plugins/plugins.ts
function defineComponentPlugin(id, options) {
  return { ...options, id, type: "component" };
}
__name(defineComponentPlugin, "defineComponentPlugin");
function sortPlugins(client, plugins) {
  const components = /* @__PURE__ */ new Map();
  function exists(id) {
    return components.has(id);
  }
  __name(exists, "exists");
  for (const plugin of plugins) {
    if (exists(plugin.id)) {
      client.log.warn("Ignoring duplicate plugin of id:", plugin.id);
      continue;
    }
    switch (plugin.type) {
      case "component":
        components.set(plugin.id, plugin);
        break;
      default:
        throw new Error(
          `Unrecognised plugin found: "${JSON.stringify(plugin)}"`
        );
    }
  }
  return {
    components
  };
}
__name(sortPlugins, "sortPlugins");

// src/utils/logger.ts
function createLogger(client) {
  const methods = {
    log: /* @__PURE__ */ __name((...messages) => {
      console.log(
        "\x1B[1m\x1B[35m[JellyCommands Log]\x1B[22m\x1B[39m",
        ...messages
      );
    }, "log"),
    error: /* @__PURE__ */ __name((...errors) => {
      console.error(
        "\x1B[1m\x1B[31m[JellyCommands Error]\x1B[22m\x1B[39m",
        ...errors
      );
    }, "error"),
    warn: /* @__PURE__ */ __name((...messages) => {
      console.warn(
        "\x1B[1m\x1B[33m[JellyCommands Debug]\x1B[22m\x1B[39m",
        ...messages
      );
    }, "warn"),
    debug: /* @__PURE__ */ __name((...messages) => {
      if (client.joptions.debug) {
        console.debug(
          "\x1B[1m\x1B[34m[JellyCommands Debug]\x1B[22m\x1B[39m",
          ...messages
        );
      }
    }, "debug")
  };
  return Object.assign(methods.log, methods);
}
__name(createLogger, "createLogger");

// src/utils/snowflake.ts
var import_discord = require("discord.js");
var import_zod = require("zod");
function isSnowflake(id) {
  try {
    import_discord.SnowflakeUtil.deconstruct(id);
    return true;
  } catch {
    return false;
  }
}
__name(isSnowflake, "isSnowflake");
var snowflakeSchema = import_zod.z.string({
  invalid_type_error: "Snowflake ids should be given as a string",
  required_error: "Must give a valid Snowflake id"
}).min(18, { message: "Discord Snowflake ids are at least 18 chars" });

// src/utils/token.ts
function cleanToken(token) {
  return typeof token === "string" ? token.replace(/^(Bot|Bearer)\s*/i, "") : null;
}
__name(cleanToken, "cleanToken");
function clientIdFromToken(token) {
  const maybeSnowflake = Buffer.from(
    token.split(".")[0],
    "base64"
  ).toString();
  return isSnowflake(maybeSnowflake) ? maybeSnowflake : null;
}
__name(clientIdFromToken, "clientIdFromToken");
function resolveToken(client) {
  return cleanToken(client.token ?? process.env?.DISCORD_TOKEN);
}
__name(resolveToken, "resolveToken");
function resolveClientId(client) {
  if (client.user?.id) return client.user?.id;
  const token = resolveToken(client);
  if (!token) return null;
  return clientIdFromToken(token);
}
__name(resolveClientId, "resolveClientId");
function getAuthData(client) {
  const token = resolveToken(client);
  if (!token) throw new Error("No token found");
  const clientId = resolveClientId(client);
  if (!clientId) throw new Error("Invalid token provided");
  return { token, clientId };
}
__name(getAuthData, "getAuthData");

// src/components/loader.ts
var import_node_path = require("path");

// src/components/components.ts
var COMPONENT_SYMBOL = Symbol.for("jellycommands.component");
var Component = class {
  static {
    __name(this, "Component");
  }
  [COMPONENT_SYMBOL];
  /**
   * The base class for all components.
   *
   * @param id Machine readable identifier for the component. Changing this is a breaking change.
   * @param name A human readable name of the component. Changing this is not considered breaking.
   */
  constructor(id, _name) {
    this[COMPONENT_SYMBOL] = id;
  }
  get id() {
    return this[COMPONENT_SYMBOL];
  }
};
function isComponent(thing) {
  return thing && typeof thing === "object" && COMPONENT_SYMBOL in thing;
}
__name(isComponent, "isComponent");

// src/components/loader.ts
var import_promises = require("fs/promises");

// src/structures/SetMap.ts
var SetMap = class {
  static {
    __name(this, "SetMap");
  }
  #map = /* @__PURE__ */ new Map();
  set(key, value) {
    this.#map.set(
      key,
      this.#map.get(key)?.add(value) || /* @__PURE__ */ new Set([value])
    );
    return this;
  }
  get(key) {
    return this.#map.get(key);
  }
  clear() {
    return this.#map.clear();
  }
  get size() {
    return this.#map.size;
  }
  delete(key, inner) {
    const current = this.#map.get(key);
    if (!current) return false;
    if (inner) {
      current?.delete(inner);
      this.#map.set(key, current);
    } else {
      this.#map.delete(key);
    }
    return true;
  }
  has(key, inner) {
    const current = this.#map.get(key);
    return !!(inner ? current?.has(inner) : current);
  }
  keys() {
    return this.#map.keys();
  }
  values() {
    return this.#map.values();
  }
  entries() {
    return this.#map.entries();
  }
};

// src/components/loader.ts
var import_node_url = require("url");
var import_node_fs = require("fs");
async function read(client, path) {
  const extensions = client.joptions.fs.extensions;
  const exists = (0, import_node_fs.existsSync)(path);
  if (!exists) {
    client.log.warn(`Given components path doesn't exist: "${path}"`);
    return [];
  }
  const isDirectory = (await (0, import_promises.stat)(path)).isDirectory();
  const files = [];
  function addFile(absolutePath, name) {
    if (name.startsWith("_")) {
      if (!isDirectory) {
        client.log.warn(
          `Given components path starts with _ and will be ignored: "${path}"`
        );
      }
      return;
    }
    if (extensions.includes((0, import_node_path.extname)(name))) {
      files.push({ absolutePath, name });
    }
  }
  __name(addFile, "addFile");
  if (isDirectory) {
    const ls = await (0, import_promises.readdir)(path, {
      withFileTypes: true,
      recursive: true
    });
    for (const file of ls) {
      if (!file.isDirectory()) {
        addFile((0, import_node_path.join)(file.parentPath, file.name), file.name);
      }
    }
  } else {
    addFile(path, (0, import_node_path.basename)(path));
  }
  return files;
}
__name(read, "read");
async function loadComponents(client, input) {
  const inputArray = Array.isArray(input) ? input : [input];
  const componentsById = new SetMap();
  const plugins = client.plugins;
  async function addComponent(component) {
    if (component.options.disabled) {
      return;
    }
    componentsById.set(component.id, component);
  }
  __name(addComponent, "addComponent");
  for (const pathOrComponent of inputArray) {
    if (typeof pathOrComponent !== "string") {
      if (isComponent(pathOrComponent)) {
        await addComponent(pathOrComponent);
      } else {
        client.log.warn(
          "A given components item is not a component or path"
        );
      }
      continue;
    }
    const files = await read(client, (0, import_node_path.resolve)(pathOrComponent));
    if (files.length === 0) {
      client.log.warn(
        "Found no components at a given path:",
        pathOrComponent
      );
    }
    for (const file of files) {
      const mod = await import((0, import_node_url.pathToFileURL)(file.absolutePath).href);
      for (const maybeComponent of Object.values(mod)) {
        if (isComponent(maybeComponent)) {
          await addComponent(maybeComponent);
        }
      }
    }
  }
  for (const [id, components] of componentsById.entries()) {
    const plugin = plugins.components.get(id);
    if (!plugin) {
      throw new Error(`Unable to find component plugin for "${id}"`);
    }
    await plugin.register(client, components);
  }
}
__name(loadComponents, "loadComponents");

// src/JellyCommands.ts
var import_v103 = require("discord-api-types/v10");
var import_ofetch = require("ofetch");

// src/structures/Cache.ts
var import_node_fs2 = require("fs");
var import_node_path2 = require("path");
var Cache = class {
  static {
    __name(this, "Cache");
  }
  name;
  cacheDirectory;
  cacheFile;
  constructor(name) {
    this.name = name;
    this.cacheDirectory = (0, import_node_path2.resolve)(".jellycommands");
    this.cacheFile = (0, import_node_path2.join)(this.cacheDirectory, `${this.name}.json`);
    if (!(0, import_node_fs2.existsSync)(this.cacheDirectory)) (0, import_node_fs2.mkdirSync)(this.cacheDirectory);
    if (!(0, import_node_fs2.existsSync)(this.cacheFile))
      (0, import_node_fs2.writeFileSync)(this.cacheFile, "{}", "utf-8");
  }
  set(object) {
    const json = JSON.stringify(object, null, 4);
    (0, import_node_fs2.writeFileSync)(this.cacheFile, json, "utf-8");
  }
  get() {
    const json = (0, import_node_fs2.readFileSync)(this.cacheFile, "utf-8");
    try {
      const object = JSON.parse(json);
      return object;
    } catch {
      return null;
    }
  }
};

// src/components/commands/register.ts
var import_v10 = require("discord-api-types/v10");
var registerCommands = /* @__PURE__ */ __name(async (client, { globalCommands, guildCommands }) => {
  const globalCommandsArray = Array.from(globalCommands);
  const commandIdMap = /* @__PURE__ */ new Map();
  const { clientId } = getAuthData(client);
  const registeredGlobalCommands = await client.$fetch(import_v10.Routes.applicationCommands(clientId), {
    method: "PUT",
    body: globalCommandsArray.map((c) => c.applicationCommandData)
  });
  registeredGlobalCommands.forEach(
    (command2, i) => commandIdMap.set(command2.id, globalCommandsArray[i])
  );
  for (const [guildId, commands] of guildCommands) {
    const commandsArray = Array.from(commands);
    const res = await client.$fetch(
      import_v10.Routes.applicationGuildCommands(clientId, guildId),
      {
        method: "PUT",
        body: commandsArray.map((c) => c.applicationCommandData)
      }
    );
    res.forEach(
      (command2, i) => commandIdMap.set(command2.id, commandsArray[i])
    );
  }
  return commandIdMap;
}, "registerCommands");

// src/components/commands/cache.ts
async function getCommandIdMap(client, commands) {
  if (!client.joptions.cache) {
    client.log.debug("Cache Disabled, registering Commands");
    return await registerCommands(client, commands);
  }
  client.log.debug("Loading Cache");
  const idResolver = new CommandIdResolver();
  const cache = new CommandCache();
  if (cache.validate(commands.commands)) {
    client.log.debug("Cache is valid");
    const commandIdMap2 = idResolver.get(commands.commands);
    if (commandIdMap2) {
      client.log.debug("Command Id Resolution success");
      return commandIdMap2;
    }
    client.log.debug("Id Resolver failed, reregistering commands");
  }
  client.log.debug("Cache is invalid, registering commands");
  const commandIdMap = await registerCommands(client, commands);
  client.log.debug("Cache Updated");
  cache.set(commands.commands);
  client.log.debug("Id Resolver Updated");
  idResolver.set(commandIdMap);
  return commandIdMap;
}
__name(getCommandIdMap, "getCommandIdMap");
var CommandCache = class {
  static {
    __name(this, "CommandCache");
  }
  cache = new Cache("command-cache");
  set(commands) {
    this.cache.set(this.getHashIds(commands));
  }
  getHashIds(commands) {
    const hashIds = [];
    for (const command2 of commands) {
      hashIds.push(command2.hashId);
    }
    return hashIds;
  }
  validate(commands) {
    const ids = this.cache.get();
    if (!ids || !Array.isArray(ids)) return false;
    const newIds = this.getHashIds(commands);
    if (newIds.length !== ids.length) return false;
    for (const id of newIds) {
      if (!ids.includes(id)) return false;
    }
    return true;
  }
};
var CommandIdResolver = class {
  static {
    __name(this, "CommandIdResolver");
  }
  cache = new Cache("command-id-resolver");
  set(commands) {
    const data = {};
    for (const [commandId, command2] of commands) {
      data[commandId] = command2.hashId;
    }
    this.cache.set(data);
  }
  get(commands) {
    const ids = this.cache.get();
    if (!ids) return false;
    const commandIdMap = /* @__PURE__ */ new Map();
    const commandsArray = [...commands];
    for (const [commandId, hashId] of Object.entries(ids)) {
      const command2 = commandsArray.find(
        (command3) => command3.hashId === hashId
      );
      if (!command2) return false;
      commandIdMap.set(commandId, command2);
    }
    return commandIdMap;
  }
};

// src/components/commands/types/options.ts
var import_zod2 = require("zod");
var baseCommandSchema = import_zod2.z.object({
  name: import_zod2.z.string(),
  nameLocalizations: import_zod2.z.object({}).catchall(import_zod2.z.string()).optional(),
  dev: import_zod2.z.boolean().default(false).optional(),
  defer: import_zod2.z.union([
    import_zod2.z.boolean().default(false),
    import_zod2.z.object({
      ephemeral: import_zod2.z.boolean().optional(),
      fetchReply: import_zod2.z.boolean().optional()
    })
  ]).optional(),
  guards: import_zod2.z.object({
    permissions: import_zod2.z.any()
  }).optional(),
  guilds: snowflakeSchema.array().nonempty().optional(),
  global: import_zod2.z.boolean().default(false).optional(),
  dm: import_zod2.z.boolean().default(false).optional(),
  disabled: import_zod2.z.boolean().default(false).optional()
});

// src/components/commands/types/commands/options.ts
var import_zod3 = require("zod");
var commandSchema = baseCommandSchema.extend({
  name: import_zod3.z.string().regex(
    // https://discord.com/developers/docs/interactions/application-commands#application-command-object
    /^[-_\p{L}\p{N}\p{sc=Deva}\p{sc=Thai}]{1,32}$/u,
    "Slash command name must be all lowercase, alphanumeric, and at most 32 chars long"
  ).refine(
    (name) => name.toLocaleLowerCase() === name,
    "Name must use lowercase chars where possible"
  ),
  description: import_zod3.z.string({ required_error: "Slash command description is required" }).min(1, "Slash command description must be at least 1 char long").max(100, "Slash command description cannot exceed 100 chars"),
  descriptionLocalizations: import_zod3.z.object({}).catchall(import_zod3.z.string()).optional(),
  options: import_zod3.z.any().array().optional()
});

// src/components/commands/types/commands/Command.ts
var import_v102 = require("discord-api-types/v10");
var import_discord3 = require("discord.js");

// src/components/commands/types/BaseCommand.ts
var import_discord2 = require("discord.js");

// src/utils/zod.ts
function parseSchema(name, schema, data) {
  const result = schema.safeParse(data);
  if (result.success) return result.data;
  const formattedError = result.error.errors.map((e) => `    => [${e.path.join(" -> ")}] (${e.code}) ${e.message}`).join("\n");
  throw new TypeError(
    `Error parsing schema for ${name}:
${formattedError}
`
  );
}
__name(parseSchema, "parseSchema");

// src/components/commands/types/BaseCommand.ts
var import_node_crypto = require("crypto");
var BaseCommand = class extends Component {
  static {
    __name(this, "BaseCommand");
  }
  /**
   * The options you pass to when creating this command.
   */
  options;
  /**
   * The callback function to call when your command is executed.
   */
  run;
  constructor({
    run,
    options,
    schema
  }) {
    super(COMMAND_COMPONENT_ID, "todo");
    if (!run || typeof run !== "function")
      throw new TypeError(
        `Expected type function for run, received ${typeof run}`
      );
    this.options = parseSchema("command", schema, options);
    this.run = run;
    if (!this.options.guilds?.length && !this.options.global && !this.options.dev)
      throw new Error(
        "Command must have at least one of guild, global, or dev"
      );
    if (this.options.global && !this.options.guilds?.length && this.options.guards) {
      throw new Error(
        "If using guards on a global command you must have a guilds array, guards can only be applied to guilds"
      );
    }
  }
  /**
   * This is for internal use and is subject to change.
   */
  get applicationCommandData() {
    return {
      name: this.options.name,
      type: this.type,
      description: "",
      default_member_permissions: this.applicationCommandPermissions,
      dm_permission: this.options.dm ?? true,
      name_localizations: this.options.nameLocalizations
    };
  }
  /**
   * This is for internal use and is subject to change.
   */
  get applicationCommandPermissions() {
    if (this.options.guards?.permissions) {
      const { bitfield } = new import_discord2.PermissionsBitField(
        this.options.guards.permissions
      );
      return bitfield.toString();
    }
    return null;
  }
  /**
   * This is for internal use and is subject to change.
   */
  get hashId() {
    return (0, import_node_crypto.createHash)("sha256").update(JSON.stringify(this.applicationCommandData)).digest("hex");
  }
};

// src/components/commands/types/commands/Command.ts
var import_discord4 = require("discord.js");
var Command = class _Command extends BaseCommand {
  static {
    __name(this, "Command");
  }
  type = import_v102.ApplicationCommandType.ChatInput;
  /**
   * On some options you can enable autocomplete, which allows you to
   * provide suggestions as a user types your command option.
   *
   * @see https://jellycommands.dev/components/commands/slash#autocomplete
   */
  autocomplete;
  constructor({
    run,
    options,
    autocomplete
  }) {
    super({ run, options, schema: commandSchema });
    if (autocomplete && typeof autocomplete !== "function") {
      throw new TypeError("Autocomplete handler must be a function");
    }
    this.autocomplete = autocomplete;
  }
  /**
   * This is for internal use and is subject to change.
   */
  static transformOptionType(option) {
    const type = typeof option.type === "string" ? import_discord4.ApplicationCommandOptionType[option.type] : option.type;
    if (!type)
      throw new Error(
        `Unable to find Slash Command Option type "${option.type}", see https://discord-api-types.dev/api/discord-api-types-v10/enum/ApplicationCommandOptionType`
      );
    let options;
    if ("options" in option && Array.isArray(option.options)) {
      options = option.options?.map(
        (o) => _Command.transformOptionType(o)
      );
    }
    return { ...option, options, type };
  }
  /**
   * This is for internal use and is subject to change.
   */
  static transformOption(option) {
    const patched = _Command.transformOptionType(option);
    const transform = (
      // @ts-expect-error todo
      import_discord3.ApplicationCommand.transformOption.bind(import_discord3.ApplicationCommand)
    );
    return transform(patched, false);
  }
  get applicationCommandData() {
    const options = this.options.options?.map(
      (o) => _Command.transformOption(o)
    );
    return {
      ...super.applicationCommandData,
      description: this.options.description,
      description_localizations: this.options.descriptionLocalizations,
      options
    };
  }
};
var command = /* @__PURE__ */ __name((options) => {
  const { run, autocomplete, ...rest } = options;
  return new Command({
    run,
    autocomplete,
    options: rest
  });
}, "command");

// src/components/commands/respond.ts
async function respond(data) {
  const { client, interaction, commandIdMap } = data;
  const isCommandOrAutocomplete = interaction.isCommand() || interaction.isContextMenuCommand() || interaction.isAutocomplete();
  if (!isCommandOrAutocomplete) return;
  const command2 = commandIdMap.get(interaction.commandId);
  if (!command2) {
    if (!interaction.isAutocomplete() && client.joptions.messages?.unknownCommand) {
      await interaction.reply(client.joptions.messages.unknownCommand);
    }
    return;
  }
  const options = command2.options;
  if (interaction.isAutocomplete()) {
    if (command2 instanceof Command) {
      await command2.autocomplete?.({
        interaction,
        client
      });
    }
    return;
  }
  if (options.defer) {
    await interaction.deferReply(
      typeof options.defer === "object" ? options.defer : {}
    );
  }
  try {
    await command2.run({ client, interaction, props: client.props });
  } catch (e) {
    console.error(
      `There was an error running command ${command2.options.name}`,
      `interaction: ${interaction.id}, channel: ${interaction.channel}, guild: ${interaction.guild}`,
      e
    );
  }
}
__name(respond, "respond");

// src/components/commands/plugin.ts
async function sortCommands(client, commands) {
  const globalCommands = /* @__PURE__ */ new Set();
  const guildCommands = /* @__PURE__ */ new Map();
  for (const command2 of commands) {
    const devMode = client.joptions.dev?.global || command2.options.dev;
    const devGuilds = client.joptions.dev?.guilds || [];
    if (devMode) {
      command2.options.dev = true;
      command2.options.global = false;
      command2.options.guilds = devGuilds;
    }
    if (command2.options.global) {
      globalCommands.add(command2);
    }
    if (command2.options.guilds) {
      for (const guildId of command2.options.guilds) {
        const existing = guildCommands.get(guildId) || /* @__PURE__ */ new Set();
        existing.add(command2);
        guildCommands.set(guildId, existing);
      }
    }
  }
  return {
    guildCommands,
    globalCommands,
    commands
  };
}
__name(sortCommands, "sortCommands");
var COMMAND_COMPONENT_ID = "jellycommands.command";
var commandsPlugin = defineComponentPlugin(
  COMMAND_COMPONENT_ID,
  {
    async register(client, _commands) {
      const commands = await sortCommands(client, _commands);
      const commandIdMap = await getCommandIdMap(client, commands);
      if (!client.joptions.dev?.guilds?.length) {
        const hasDevCommand = Array.from(commands.commands).some(
          (command2) => command2.options.dev
        );
        if (client.joptions.dev?.global || hasDevCommand) {
          throw new Error(
            "You must provide at least one guild id in the dev guilds array to use dev commands"
          );
        }
      }
      client.on("interactionCreate", (interaction) => {
        client.log.debug(`Interaction received: ${interaction.id} | ${interaction.type} | Command Id: ${interaction.isCommand() && interaction.commandId}`);
        respond({
          interaction,
          client,
          commandIdMap
        });
      });
    }
  }
);

// src/components/buttons/plugin.ts
var BUTTONS_COMPONENT_ID = "jellycommands.button";
async function findButton(incomingId, buttons) {
  for (const button2 of buttons) {
    const { id } = button2.options;
    switch (typeof id) {
      case "string":
        if (id === incomingId) return button2;
        break;
      case "function":
        if (await id(incomingId)) return button2;
        break;
      case "object":
        if (id.test(incomingId)) return button2;
        break;
    }
  }
  return null;
}
__name(findButton, "findButton");
var buttonsPlugin = defineComponentPlugin(
  BUTTONS_COMPONENT_ID,
  {
    register(client, buttons) {
      client.on("interactionCreate", async (interaction) => {
        if (interaction.isButton()) {
          const button2 = await findButton(
            interaction.customId,
            buttons
          );
          if (button2) {
            if (button2.options.defer) {
              await interaction.deferReply(
                typeof button2.options.defer === "object" ? button2.options.defer : {}
              );
            }
            await button2.run({
              client,
              props: client.props,
              interaction
            });
          }
        }
      });
    }
  }
);

// src/components/events/plugin.ts
var EVENTS_COMPONENT_ID = "jellycommands.event";
var eventsPlugin = defineComponentPlugin(EVENTS_COMPONENT_ID, {
  register(client, events) {
    for (const event2 of events) {
      async function cb(...ctx) {
        try {
          await event2.run({ client, props: client.props }, ...ctx);
        } catch (error) {
          console.error(
            `There was an error running event ${event2.name}`,
            error
          );
        }
      }
      __name(cb, "cb");
      event2.options.once ? client.once(event2.name, cb) : client.on(event2.name, cb);
    }
  }
});

// src/components/modals/plugin.ts
var MODALS_COMPONENT_ID = "jellycommands.modal";
async function findModal(incomingId, modals) {
  for (const modal2 of modals) {
    const { id } = modal2.options;
    switch (typeof id) {
      case "string":
        if (id === incomingId) return modal2;
        break;
      case "function":
        if (await id(incomingId)) return modal2;
        break;
      case "object":
        if (id.test(incomingId)) return modal2;
        break;
    }
  }
  return null;
}
__name(findModal, "findModal");
var modalsPlugin = defineComponentPlugin(MODALS_COMPONENT_ID, {
  register(client, modals) {
    client.on("interactionCreate", async (interaction) => {
      if (interaction.isModalSubmit()) {
        const modal2 = await findModal(interaction.customId, modals);
        if (modal2) {
          if (modal2.options.defer) {
            await interaction.deferReply(
              typeof modal2.options.defer === "object" ? modal2.options.defer : {}
            );
          }
          await modal2.run({
            client,
            props: client.props,
            interaction
          });
        }
      }
    });
  }
});

// src/plugins/core.ts
var CORE_PLUGINS = [
  buttonsPlugin,
  commandsPlugin,
  eventsPlugin,
  modalsPlugin
];

// src/JellyCommands.ts
var import_discord5 = require("discord.js");

// src/options.ts
var import_zod5 = require("zod");
var jellyCommandsOptionsSchema = import_zod5.z.object({
  components: import_zod5.z.union([
    import_zod5.z.string(),
    import_zod5.z.array(
      import_zod5.z.union([
        import_zod5.z.string(),
        import_zod5.z.any().refine(
          (component) => isComponent(component),
          "Should be a component instance"
        )
      ])
    )
  ]).optional(),
  clientOptions: import_zod5.z.object({}).passthrough(),
  props: import_zod5.z.object({}).passthrough().default({}),
  // plugins: z.array(z.object({}).passthrough()).optional(),
  messages: import_zod5.z.object({
    unknownCommand: import_zod5.z.union([
      import_zod5.z.string(),
      import_zod5.z.object({}).passthrough().default({ embeds: [{ description: "Unknown Command" }] })
    ])
  }).default({}),
  dev: import_zod5.z.object({
    global: import_zod5.z.boolean().default(false),
    guilds: snowflakeSchema.array().nonempty().optional()
  }).default({}),
  cache: import_zod5.z.boolean().default(true),
  debug: import_zod5.z.boolean().default(() => !!process.env.DEBUG),
  fs: import_zod5.z.object({
    extensions: import_zod5.z.string().array().default([".js", ".ts"])
  }).default({})
});

// src/JellyCommands.ts
var JellyCommands = class extends import_discord5.Client {
  static {
    __name(this, "JellyCommands");
  }
  // todo these options include data that isn't relevant (like given components) so mayb shouldn't be here
  /**
   * The options you pass to JellyCommands when creating your client.
   */
  joptions;
  /**
   * todo https://github.com/ghostdevv/jellycommands/issues/209
   */
  plugins;
  /**
   * Props are used to pass data around your client.
   * @see https://jellycommands.dev/components/props/
   */
  props;
  /**
   * A wrapper of console.log that adds a nice prefix.
   * Designed for internal use, so won't expand upon current functionality.
   */
  log;
  constructor(options) {
    super(options.clientOptions);
    this.joptions = parseSchema(
      "JellyCommands options",
      jellyCommandsOptionsSchema,
      options
    );
    this.log = createLogger(this);
    this.props = options.props || {};
    this.plugins = sortPlugins(this, CORE_PLUGINS);
    this.props = {
      get() {
        throw new Error(
          "props.get has been removed, SEE: https://jellycommands.dev/migrate/props"
        );
      },
      set() {
        throw new Error(
          "props.set has been removed, SEE: https://jellycommands.dev/migrate/props"
        );
      },
      has() {
        throw new Error(
          "props.has has been removed, SEE: https://jellycommands.dev/migrate/props"
        );
      },
      ...this.props
    };
  }
  /**
   * Internal fetch wrapper for making Discord API requests.
   * Please don't use this, as it may have breaking changes or disappear.
   */
  async $fetch(path, options) {
    return await (0, import_ofetch.ofetch)(path, {
      baseURL: import_v103.RouteBases.api,
      headers: {
        ...options?.headers || {},
        Authorization: `Bot ${this.token}`
      },
      method: options?.method,
      body: options?.body,
      query: options?.query || void 0,
      retry: 5,
      retryDelay: 500,
      onResponseError: /* @__PURE__ */ __name((context) => {
        if (context.options.retryStatusCodes?.includes(
          context.response.status
        )) {
          this.log.debug(`[Discord Request Rate Limited] Retrying in ${context.options.retryDelay}ms`);
        }
        const contextStr = typeof context.response?._data === "object" ? `:
${JSON.stringify(context.response._data, null, 2)}
` : " NO CONTEXT RETURNED";
        this.log.error(
          `[Discord Fetch Error (${context.response?.status})]${contextStr}`
        );
      }, "onResponseError")
    });
  }
  /**
   * This must be called once in order to start your Discord Bot.
   * It initialises all JellyCommands specific code, then calls
   * the base discord.js `client.login()` fn to run your bot.
   */
  async login(potentialToken) {
    this.token ||= cleanToken(potentialToken);
    const token = resolveToken(this);
    if (!token) {
      throw new Error("No bot token was found");
    }
    if (this.joptions.components?.length) {
      this.log.debug("Loading components");
      await loadComponents(this, this.joptions.components);
    } else {
      this.log.debug("No components given");
    }
    return super.login(token);
  }
};

// src/components/commands/types/messageCommands/options.ts
var messageCommandSchema = baseCommandSchema.extend({});

// src/components/commands/types/messageCommands/MessageCommand.ts
var import_v104 = require("discord-api-types/v10");
var MessageCommand = class extends BaseCommand {
  static {
    __name(this, "MessageCommand");
  }
  type = import_v104.ApplicationCommandType.Message;
  constructor(run, options) {
    super({ run, options, schema: messageCommandSchema });
  }
};
var messageCommand = /* @__PURE__ */ __name((options) => {
  const { run, ...rest } = options;
  return new MessageCommand(run, rest);
}, "messageCommand");

// src/components/commands/types/userCommands/options.ts
var userCommandSchema = baseCommandSchema.extend({});

// src/components/commands/types/userCommands/UserCommand.ts
var import_v105 = require("discord-api-types/v10");
var UserCommand = class extends BaseCommand {
  static {
    __name(this, "UserCommand");
  }
  type = import_v105.ApplicationCommandType.User;
  constructor(run, options) {
    super({ run, options, schema: userCommandSchema });
  }
};
var userCommand = /* @__PURE__ */ __name((options) => {
  const { run, ...rest } = options;
  return new UserCommand(run, rest);
}, "userCommand");

// src/components/events/options.ts
var import_zod7 = require("zod");
var eventSchema = import_zod7.z.object({
  name: import_zod7.z.string(),
  disabled: import_zod7.z.boolean().default(false),
  once: import_zod7.z.boolean().default(false)
});

// src/components/events/Event.ts
var Event = class extends Component {
  constructor(name, run, _options) {
    super(EVENTS_COMPONENT_ID, "Event");
    this.name = name;
    this.run = run;
    if (!name || typeof name !== "string")
      throw new TypeError(
        `Expected type string for name, received ${typeof name}`
      );
    if (!run || typeof run !== "function")
      throw new TypeError(
        `Expected type function for run, received ${typeof run}`
      );
    this.options = parseSchema("event options", eventSchema, _options);
  }
  static {
    __name(this, "Event");
  }
  /**
   * The options you pass to when creating this event.
   */
  options;
  /**
   * Type narrowing utility to check whether something is an event.
   * @param item The thing to check
   */
  static is(item) {
    return isComponent(item) && item.id === EVENTS_COMPONENT_ID;
  }
};
var event = /* @__PURE__ */ __name((options) => {
  const { run, ...rest } = options;
  return new Event(options.name, run, rest);
}, "event");

// src/components/buttons/options.ts
var import_zod9 = require("zod");
var buttonSchema = import_zod9.z.object({
  id: import_zod9.z.union([
    import_zod9.z.string(),
    import_zod9.z.instanceof(RegExp),
    // todo test this
    import_zod9.z.function().args(import_zod9.z.string().optional()).returns(import_zod9.z.union([import_zod9.z.boolean(), import_zod9.z.promise(import_zod9.z.boolean())]))
  ]),
  defer: import_zod9.z.union([
    import_zod9.z.boolean().default(false),
    import_zod9.z.object({
      ephemeral: import_zod9.z.boolean().optional(),
      fetchReply: import_zod9.z.boolean().optional()
    })
  ]).optional(),
  disabled: import_zod9.z.boolean().default(false).optional()
});

// src/components/buttons/buttons.ts
var Button = class extends Component {
  constructor(_options, run) {
    super(BUTTONS_COMPONENT_ID, "Button");
    this.run = run;
    this.options = parseSchema("button", buttonSchema, _options);
  }
  static {
    __name(this, "Button");
  }
  options;
  static is(item) {
    return isComponent(item) && item.id === BUTTONS_COMPONENT_ID;
  }
};
var button = /* @__PURE__ */ __name((options) => {
  const { run, ...rest } = options;
  return new Button(rest, run);
}, "button");

// src/components/modals/options.ts
var import_zod11 = require("zod");
var modalSchema = import_zod11.z.object({
  id: import_zod11.z.union([
    import_zod11.z.string(),
    import_zod11.z.instanceof(RegExp),
    // todo test this
    import_zod11.z.function().args(import_zod11.z.string().optional()).returns(import_zod11.z.union([import_zod11.z.boolean(), import_zod11.z.promise(import_zod11.z.boolean())]))
  ]),
  defer: import_zod11.z.union([
    import_zod11.z.boolean().default(false),
    import_zod11.z.object({
      ephemeral: import_zod11.z.boolean().optional(),
      fetchReply: import_zod11.z.boolean().optional()
    })
  ]).optional(),
  disabled: import_zod11.z.boolean().default(false).optional()
});

// src/components/modals/modals.ts
var Modal = class extends Component {
  constructor(_options, run) {
    super(MODALS_COMPONENT_ID, "Modal");
    this.run = run;
    this.options = parseSchema("modal", modalSchema, _options);
  }
  static {
    __name(this, "Modal");
  }
  options;
  static is(item) {
    return isComponent(item) && item.id === MODALS_COMPONENT_ID;
  }
};
var modal = /* @__PURE__ */ __name((options) => {
  const { run, ...rest } = options;
  return new Modal(rest, run);
}, "modal");
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  Button,
  Command,
  Event,
  JellyCommands,
  MessageCommand,
  Modal,
  UserCommand,
  button,
  command,
  event,
  messageCommand,
  modal,
  userCommand
});
